@@ -16,19 +16,21 @@ from pathlib import Path
 from typing import List, Optional, Dict, Set
 from urllib.parse import ParseResult as URL
 
+from dataclasses import dataclass
+
 from rlbot.gamelaunch.epic_launch import launch_with_epic_login_trick, launch_with_epic_simple
 from rlbot.utils.structures import game_data_struct
 
 from rlbot import gateway_util
 from rlbot import version
 from rlbot.base_extension import BaseExtension
-from rlbot.botmanager.bot_manager_flatbuffer import BotManagerFlatbuffer
 from rlbot.botmanager.bot_manager_independent import BotManagerIndependent
 from rlbot.botmanager.bot_manager_struct import BotManagerStruct
 from rlbot.botmanager.helper_process_manager import HelperProcessManager
+from rlbot.botmanager.agent_metadata import AgentMetadata
 from rlbot.gateway_util import LaunchOptions, NetworkingRole
 from rlbot.matchconfig.conversions import parse_match_config
-from rlbot.matchconfig.match_config import MatchConfig
+from rlbot.matchconfig.match_config import MatchConfig, PlayerConfig
 from rlbot.matchconfig.psyonix_config import set_random_psyonix_bot_preset
 from rlbot.matchcomms.server import launch_matchcomms_server
 from rlbot.parsing.bot_config_bundle import get_bot_config_bundle, BotConfigBundle, get_script_config_bundle
@@ -42,6 +44,7 @@ from rlbot.utils.structures.start_match_structures import MAX_PLAYERS
 from rlbot.utils.structures.game_interface import GameInterface
 from rlbot.utils.config_parser import mergeTASystemSettings, cleanUpTASystemSettings
 from rlbot.matchcomms.server import MatchcommsServerThread
+from rlbot.utils.virtual_environment_management import EnvBuilderWithRequirements
 
 if platform.system() == 'Windows':
     import msvcrt
@@ -56,13 +59,12 @@ class ROCKET_LEAGUE_PROCESS_INFO:
     PROGRAM_NAME = 'RocketLeague.exe'
     PROGRAM = 'RocketLeague.exe'
     REQUIRED_ARGS = {r'-rlbot', r'RLBot_ControllerURL=127.0.0.1:[0-9]+'}
-    PORT_PLACEHOLDER = '%PORT%'
-    IDEAL_ARGS = ['-rlbot', f'RLBot_ControllerURL=127.0.0.1:{PORT_PLACEHOLDER}', '-nomovie']
 
     @staticmethod
     def get_ideal_args(port):
-        return [arg.replace(ROCKET_LEAGUE_PROCESS_INFO.PORT_PLACEHOLDER, str(port))
-                for arg in ROCKET_LEAGUE_PROCESS_INFO.IDEAL_ARGS]
+        # We are specifying RLBot_PacketSendRate=240, which will override people's TARLBot.ini settings.
+        # We believe there is no downside to 240. See https://github.com/RLBot/RLBot/wiki/Tick-Rate
+        return ['-rlbot', f'RLBot_ControllerURL=127.0.0.1:{port}', 'RLBot_PacketSendRate=240', '-nomovie']
 
 @dataclass
 class RocketLeagueLauncherPreference:
@@ -98,6 +100,18 @@ def setup_manager_context():
         setup_manager.shut_down(kill_all_pids=True)
 
 
+@dataclass
+class BotProcessInfo:
+    process: mp.Process
+    subprocess: subprocess.Popen
+    player_config: PlayerConfig
+
+    def is_alive(self):
+        if self.process is not None:
+            return self.process.is_alive()
+        return self.subprocess.poll() is None
+
+
 class SetupManager:
     """
     This class is responsible for pulling together all bits of the framework to
@@ -123,7 +137,7 @@ class SetupManager:
     start_match_configuration = None
     agent_metadata_queue = mp.Queue()
     extension = None
-    bot_processes: Dict[int, mp.Process] = {}
+    bot_processes: Dict[int, BotProcessInfo] = {}
     script_processes: Dict[int, subprocess.Popen] = {}
 
     def __init__(self):
@@ -133,7 +147,7 @@ class SetupManager:
         self.helper_process_manager = HelperProcessManager(self.quit_event)
         self.bot_quit_callbacks = []
         self.bot_reload_requests = []
-        self.agent_metadata_map = {}
+        self.agent_metadata_map: Dict[int, AgentMetadata] = {}
         self.match_config: MatchConfig = None
         self.rlbot_gateway_process = None
         self.matchcomms_server: MatchcommsServerThread = None
@@ -186,7 +200,11 @@ class SetupManager:
             self.launch_rocket_league(port=port, launcher_preference=launcher_preference)
 
         try:
-            self.game_interface.load_interface()
+            self.logger.info("Loading interface...")
+            # We're not going to use this game_interface for much, just sending start match messages and inspecting
+            # the packet to see if the appropriate cars have been spawned.
+            self.game_interface.load_interface(
+                port=23234, wants_ball_predictions=False, wants_quick_chat=False, wants_game_messages=False)
         except Exception as e:
             self.logger.error("Terminating rlbot gateway and raising:")
             self.rlbot_gateway_process.terminate()
@@ -244,10 +262,12 @@ class SetupManager:
 
             try:
                 _ = subprocess.Popen(linux_args)
+                return
             except OSError:
-                self.logger.warning('Could not find Steam executable, using browser to open instead.')
+                self.logger.warning('Could not launch Steam executable on Linux.')
 
         try:
+            self.logger.info("Launching rocket league via steam browser URL as a last resort...")
             webbrowser.open(f'steam://rungameid/{ROCKET_LEAGUE_PROCESS_INFO.GAMEID}//{args_string}')
         except webbrowser.Error:
             self.logger.warning(
@@ -285,6 +305,17 @@ class SetupManager:
         if match_config.extension_config is not None and match_config.extension_config.python_file_path is not None:
             self.load_extension(match_config.extension_config.python_file_path)
 
+        for bundle in self.bot_bundles:
+            if bundle is not None and bundle.use_virtual_environment:
+                builder = EnvBuilderWithRequirements(bundle=bundle)
+                builder.create(Path(bundle.config_directory) / 'venv')
+
+        for script_config in match_config.script_configs:
+            script_config_bundle = get_script_config_bundle(script_config.config_path)
+            if script_config_bundle.use_virtual_environment:
+                builder = EnvBuilderWithRequirements(bundle=script_config_bundle)
+                builder.create(Path(script_config_bundle.config_directory) / 'venv')
+
         self.match_config = match_config
         self.start_match_configuration = match_config.create_match_settings()
         self.game_interface.start_match_configuration = self.start_match_configuration
@@ -348,7 +379,7 @@ class SetupManager:
         expected_metadata_calls = sum(1 for player in self.match_config.player_configs if player.rlbot_controlled)
         return self.num_metadata_received >= expected_metadata_calls
 
-    def launch_early_start_bot_processes(self):
+    def launch_early_start_bot_processes(self, match_config: MatchConfig = None):
         """
         Some bots can start up before the game is ready and not be bothered by missing
         or strange looking values in the game tick packet, etc. Such bots can opt in to the
@@ -362,7 +393,7 @@ class SetupManager:
             return  # The bot indices are liable to change, so don't start anything yet.
 
         self.logger.debug("Launching early-start bot processes")
-        num_started = self.launch_bot_process_helper(early_starters_only=True)
+        num_started = self.launch_bot_process_helper(early_starters_only=True, match_config=match_config or self.match_config)
         self.try_recieve_agent_metadata()
         if num_started > 0 and self.early_start_seconds > 0:
             self.logger.info(f"Waiting for {self.early_start_seconds} seconds to let early-start bots load.")
@@ -371,11 +402,11 @@ class SetupManager:
                 self.try_recieve_agent_metadata()
                 time.sleep(0.1)
 
-    def launch_bot_processes(self):
+    def launch_bot_processes(self, match_config: MatchConfig = None):
         self.logger.debug("Launching bot processes")
-        self.launch_bot_process_helper(early_starters_only=False)
+        self.launch_bot_process_helper(early_starters_only=False, match_config=match_config or self.match_config)
 
-    def launch_bot_process_helper(self, early_starters_only=False):
+    def launch_bot_process_helper(self, early_starters_only=False, match_config: MatchConfig = None):
         # Start matchcomms here as it's only required for the bots.
         self.kill_matchcomms_server()
         self.matchcomms_server = launch_matchcomms_server()
@@ -389,15 +420,15 @@ class SetupManager:
         self.game_interface.update_live_data_packet(packet)
 
         # TODO: root through the packet and find discrepancies in the player index mapping.
-        for i in range(min(self.num_participants, len(self.match_config.player_configs))):
+        for i in range(min(self.num_participants, len(match_config.player_configs))):
 
-            player_config = self.match_config.player_configs[i]
+            player_config = match_config.player_configs[i]
             if not player_config.has_bot_script():
                 continue
             if early_starters_only and not self.bot_bundles[i].supports_early_start:
                 continue
 
-            spawn_id = self.game_interface.start_match_configuration.player_configuration[i].spawn_id
+            spawn_id = player_config.spawn_id
 
             if early_starters_only:
                 # Danger: we have low confidence in this since we're not leveraging the spawn id.
@@ -412,31 +443,62 @@ class SetupManager:
                         self.logger.info(f'Looks good, considering participant index to be {n}')
                         participant_index = n
                 if participant_index is None:
-                    raise Exception("Unable to determine the bot index!")
+                    for prox_index, proc_info in self.bot_processes.items():
+                        if spawn_id == proc_info.player_config.spawn_id:
+                            participant_index = prox_index
+                    if participant_index is None:
+                        raise Exception(f"Unable to determine the bot index for spawn id {spawn_id}")
 
             if participant_index not in self.bot_processes:
-                reload_request = mp.Event()
-                quit_callback = mp.Event()
-                self.bot_reload_requests.append(reload_request)
-                self.bot_quit_callbacks.append(quit_callback)
-                process = mp.Process(target=SetupManager.run_agent,
-                                     args=(self.quit_event, quit_callback, reload_request, self.bot_bundles[i],
-                                           str(self.start_match_configuration.player_configuration[i].name),
-                                           self.teams[i], participant_index, self.python_files[i], self.agent_metadata_queue,
-                                           self.match_config, self.matchcomms_server.root_url, spawn_id))
-                process.start()
-                self.bot_processes[participant_index] = process
+                bundle = get_bot_config_bundle(player_config.config_path)
+                name = str(self.start_match_configuration.player_configuration[i].name)
+                if bundle.supports_standalone:
+                    executable = sys.executable
+                    if bundle.use_virtual_environment:
+                        executable = str(Path(bundle.config_directory) / 'venv' / 'Scripts' / 'python.exe')
+                    process = subprocess.Popen([
+                        executable,
+                        bundle.python_file,
+                        '--config-file', str(player_config.config_path),
+                        '--name', name,
+                        '--team', str(self.teams[i]),
+                        '--player-index', str(participant_index),
+                        '--spawn-id', str(spawn_id),
+                        '--matchcomms-url', self.matchcomms_server.root_url.geturl()
+                    ], cwd=Path(bundle.config_directory).parent)
+                    self.bot_processes[participant_index] = BotProcessInfo(process=None, subprocess=process, player_config=player_config)
+
+                    # Insert immediately into the agent metadata map because the standalone process has no way to communicate it back out
+                    self.agent_metadata_map[participant_index] = AgentMetadata(participant_index, name, self.teams[i], {process.pid})
+                else:
+                    reload_request = mp.Event()
+                    quit_callback = mp.Event()
+                    self.bot_reload_requests.append(reload_request)
+                    self.bot_quit_callbacks.append(quit_callback)
+                    process = mp.Process(target=SetupManager.run_agent,
+                                         args=(self.quit_event, quit_callback, reload_request, self.bot_bundles[i],
+                                               name,
+                                               self.teams[i], participant_index, self.python_files[i], self.agent_metadata_queue,
+                                               match_config, self.matchcomms_server.root_url, spawn_id))
+                    process.start()
+                    self.bot_processes[participant_index] = BotProcessInfo(process=process, subprocess=None, player_config=player_config)
                 num_started += 1
 
-        self.logger.debug(f"Successfully started {num_started} bot processes")
+        self.logger.info(f"Successfully started {num_started} bot processes")
+
+        process_configuration.configure_processes(self.agent_metadata_map, self.logger)
 
         scripts_started = 0
-        for script_config in self.match_config.script_configs:
+        for script_config in match_config.script_configs:
             script_config_bundle = get_script_config_bundle(script_config.config_path)
             if early_starters_only and not script_config_bundle.supports_early_start:
                 continue
-            process = subprocess.Popen([sys.executable, script_config_bundle.script_file],
-                                       shell=True, cwd=Path(script_config_bundle.config_directory).parent)
+            executable = sys.executable
+            if script_config_bundle.use_virtual_environment:
+                executable = str(Path(script_config_bundle.config_directory) / 'venv' / 'Scripts' / 'python.exe')
+
+            process = subprocess.Popen([executable, script_config_bundle.script_file],
+                                       cwd=Path(script_config_bundle.config_directory).parent)
             self.logger.info(f"Started script with pid {process.pid} using {process.args}")
             self.script_processes[process.pid] = process
             scripts_started += 1
@@ -525,7 +587,7 @@ class SetupManager:
         num_recieved = 0
         while True:  # will exit on queue.Empty
             try:
-                single_agent_metadata = self.agent_metadata_queue.get(timeout=0.1)
+                single_agent_metadata: AgentMetadata = self.agent_metadata_queue.get(timeout=0.1)
                 num_recieved += 1
                 self.helper_process_manager.start_or_update_helper_process(single_agent_metadata)
                 self.agent_metadata_map[single_agent_metadata.index] = single_agent_metadata
@@ -607,20 +669,18 @@ class SetupManager:
             bm = BotManagerIndependent(terminate_event, callback_event, reload_request, config_file, name, team, index,
                                        agent_class_wrapper, agent_telemetry_queue, match_config, matchcomms_root,
                                        spawn_id)
-        elif hasattr(agent_class_wrapper.get_loaded_class(), "get_output_flatbuffer"):
-            bm = BotManagerFlatbuffer(terminate_event, callback_event, reload_request, config_file, name, team, index,
-                                      agent_class_wrapper, agent_telemetry_queue, match_config, matchcomms_root,
-                                      spawn_id)
         else:
             bm = BotManagerStruct(terminate_event, callback_event, reload_request, config_file, name, team, index,
                                   agent_class_wrapper, agent_telemetry_queue, match_config, matchcomms_root, spawn_id)
         bm.run()
 
     def kill_bot_processes(self):
-        for process in self.bot_processes.values():
-            process.terminate()
-        for process in self.bot_processes.values():
-            process.join(timeout=1)
+        for process_info in self.bot_processes.values():
+            proc = process_info.process or process_info.subprocess
+            proc.terminate()
+        for process_info in self.bot_processes.values():
+            if process_info.process:
+                process_info.process.join(timeout=1)
         self.bot_processes.clear()
         self.num_metadata_received = 0
 
@@ -632,12 +692,12 @@ class SetupManager:
                     self.logger.info(f"Killing {child.pid} (child of {pid})")
                     try:
                         child.kill()
-                    except psutil._exceptions.NoSuchProcess:
+                    except psutil.NoSuchProcess:
                         self.logger.info("Already dead.")
                 self.logger.info(f"Killing {pid}")
                 try:
                     parent.kill()
-                except psutil._exceptions.NoSuchProcess:
+                except psutil.NoSuchProcess:
                     self.logger.info("Already dead.")
             except psutil.NoSuchProcess:
                 self.logger.info("Can't fetch parent process, already dead.")
